local love = love
local class = require "class"

local Button = class("Object", "Size", "Event")


function Button:ctor(t, f)
  self.name = t
  self.text = t and tostring(t) or ""
  self.font = f or love.graphics.getFont()
  self.text_w = self.font:getWidth(self.text)
  self.text_h = self.font:getHeight(self.text)
  self.image = nil
  self.align = "center"
  self.pressed = false
  self.width = self.text_w
  self.height = self.text_h
end


function Button:setText(t)
  self.name = t
  self.text = t and tostring(t) or ""
  self.text_w = self.font:getWidth(self.text)
  self.text_h = self.font:getHeight(self.text)
  if self.width < self.text_w then
    self.width = self.text_w
  end
  if self.height < self.text_h then
    self.height = self.text_h
  end
end


function Button:setFont(f)
  self.font = f
  self.text_w = f:getWidth(self.text)
  self.text_h = f:getHeight(self.text)
  if self.width < self.text_w then
    self.width = self.text_w
  end
  if self.height < self.text_h then
    self.height = self.text_h
  end
end


function Button:setImage(i)
  self.image = i
  local w = i:getWidth()
  local h = i:getHeight()
  if self.width < w then
    self.width = w
  end
  if self.height < h then
    self.height = h
  end
end


function Button:getText()
  return self.text
end


function Button:draw()
  if self.alpha == 0 then
    return
  end
  love.graphics.setFont(self.font)
  if self.image then
    love.graphics.draw(self.image, self.x, self.y)
  end
  if self.align == "center" then
    love.graphics.print(self.text, self.x + (self.width - self.text_w) / 2, self.y + (self.height - self.text_h) / 2)
  end
  if self.pressed then
    love.graphics.rectangle("fill", self.x, self.y, self.width, self.height)
  end
end


function Button:mousepressed(x, y)
  if self:focus(x, y, self.width, self.height) then
    self.pressed = true
  end
end


function Button:mousereleased(x, y)
  if self.pressed and self:focus(x, y, self.width, self.height) then
    self:toast("BUTTON_RELEASED")
  end
  self.pressed = false
end


return Button