local class = require "class"

local Scene = class("Object", "Size", "Event")

local List = require "list"

local list = List()
local loader = {}
local scene = {}
local manager = {}


function manager.load(n)
  if not loader[n] then
    loader[n] = require("scenes." .. n)
  end
  if not scene[n] then
    scene[n] = Scene(n)
    loader[n](scene[n])
  end
end


function manager.open(n)
  if not scene[n] then
    manager.load(n)
  end
  list:setBack(scene[n]) -- in setBack() list.back is correct
  
  -- here list.back can be equal to scene.Game
  -- and can be equal to scene.Setting
  -- simultaneously.
end


function manager.close(n)
  list:erase(scene[n])
  scene[n] = nil
end


function Scene:ctor(n)
  self.name = n
  self.component_list = List()
  self.pressed = false
  self.released = false
end


function Scene:addComponent(c)
  self.component_list:setBack(c)
end


function Scene:update(dt)
  local c = self.component_list:getBack()
  while c do
    if self.alpha > 0 then
      if self.pressed and c.mousepressed and c:mousepressed(self.px, self.py) then
        self.pressed = false
      end
      if self.released and c.mousereleased and c:mousereleased(self.rx, self.ry) then
        self.released = false
      end
    end
    if c.update then
      c:update(dt)
    end
    c = self.component_list:getPrevious(c)
  end
  self.pressed = false
  self.released = false
end


function Scene:draw()
  if self.alpha > 0 then
    local c = self.component_list:getFront()
    while c do
      if c.draw then
        c:draw()
      end
      c = self.component_list:getNext(c)
    end
  end
end


function Scene:mousepressed(x, y, b)
  if self:focus(x, y, self.width, self.height) then
    self.pressed = true
    self.px = x
    self.py = y
    return true
  end
end


function Scene:mousereleased(x, y, b)
  if self:focus(x, y, self.width, self.height) then
    self.released = true
    self.rx = x
    self.ry = y
    return true
  end
end



function love.update(dt)
  local s = list:getBack()
  while s do
    s:update(dt)
    s = list:getPrevious(s)
  end
end


function love.draw()
  local s = list:getFront()
  while s do
    s:draw()
    s = list:getNext(s)
  end
end


function love.mousepressed(...)
  local s = list:getBack()
  while s do
    if s:mousepressed(...) then
      focus = s
      break
    end
    s = list:getPrevious(s)
  end
end


function love.mousereleased(...)
  local s = list:getBack()
  while s do
    if s:mousereleased(...) then
      break
    end
    s = list:getPrevious(s)
  end
end


return manager